#ifndef CRITTERB_H
#define CRITTERB_H

#include "btBulletDynamicsCommon.h"
#include "GL/gl.h"

#include "../../utils/parser.h"
#include "../../utils/randgen.h"
#include "../../brainz/brainz.h"
#include "../../utils/settings.h"
#include "../../utils/displaylists.h"
#include "../../utils/raycast.h"
#include "entity.h"
#include "body.h"
#include "food.h"
#include "circuit8.h"
// #include <cmath>
// #include <vector>
// #include <iostream>
// #include <sstream>

using namespace std;

class CritterB : public Entity
{
	public:
		CritterB(btDynamicsWorld* btWorld, long unsigned int id, const btVector3& startPos, unsigned char* retinap, Circuit8* realcircuit8);
		CritterB(CritterB& , long unsigned int id, const btVector3& startPos, bool brainmutant, bool bodymutant, Circuit8* realcircuit8);
		CritterB(string &critterstring, btDynamicsWorld* btWorld, const btVector3& startPos, unsigned char* retinap, Circuit8* realcircuit8);
		~CritterB();

		Brainz			brain;
		Body			body;
		inline void		registerBrainInputOutputs();
		void			draw(bool drawFaces);
		void			move();
		
		bool			eaten;
		
		unsigned long		critterID;

		unsigned int		adamdist;
		btScalar		position[16];

/*		float			color[3];*/
		float			speciescolor[4];
		float			headcolor[4];
		float			colorTrim;

// 		unsigned int		lifetime;
		float			energyLevel;
		float			energyUsed;
		unsigned int		totalFrames;

		// Inputs
		bool			canProcreate;
		bool			touchingFood;
		bool			touchingPill;
// 		Food*			touchedFoodID;
		bool			touchingCritter;
// 		CritterB*		touchedCritterID;
		Entity*			touchedEntity;

		// Motor Func
 		bool			eat;
  		bool			procreate;
  		bool			blink;
  		bool			lockblink;
  		bool			blinkscreen;
  		bool			lockblinkscreen; // can stitch shut their minds eye as well >:O

  		bool			moreRed[64*49];
  		bool			moreGreen[64*49];
  		bool			moreBlue[64*49];
  		bool			lessRed[64*49];
		bool			lessGreen[64*49];
		bool			lessBlue[64*49];
		bool			condensedColourOp[64*49];
		bool			writeIntensity[3];
		bool			condensedMoreRed;
		bool			condensedLessRed;
		bool			condensedMoreGreen;
		bool			condensedLessGreen;
		bool			condensedMoreBlue;
		bool			condensedLessBlue;
		bool			votefastup;
		bool			votefastdown;
		bool			voteleft;
		bool			voteright;
		bool			votefire;
		bool			voteerif;
		bool			votepause;
		bool			votefastup2;
		bool			votefastdown2;
		bool			voteleft2;
		bool			voteright2;
		bool			votefire2;
		bool			voteerif2;
		bool			votepause2;
		bool			votefastup3;
		bool			votefastdown3;
		bool			voteleft3;
		bool			voteright3;
		bool			votefire3;
		bool			voteerif3;
		bool			votepause3;
		bool			votedisperse;
		bool			votedisperse2;
		bool			votedisperse3;
		bool			votedisrepse;
		bool			votedisrepse2;
		bool			votedisrepse3;
		

//		bool			binaryColourOpSelector[3];

		unsigned int		procreateTimeCount;
		unsigned int		procreateTimeTrigger;

		void			process();

		void			loadCritterB(string &content);
		string			saveCritterB();

		// Vision
		unsigned int		retinasize;
// 		float			sightrange;
		float			straal;
		unsigned char		*retina;
		unsigned int		items;
		void			place();
		void			calcFramePos(unsigned int pos);
		void			printVision();
		
		float			fitness_index;

		Circuit8*		circuit8;
		
	private:

		Settings		*settings;
		Raycast*		raycast;
		castResult		mouseRay;

// 		unsigned int		motorneuronsfired;
		unsigned int		movementsmade;

		btDynamicsWorld*	btDynWorld;

		btVector3 getScreenDirection(const int& x, const int& y);

		inline void		initConst();
		inline void		procInputNeurons();
		inline void		createInputOutputNeurons();

		inline void		mutateBody();
		inline void		mutateBrain();

		// Vision
			unsigned int		components;
			unsigned int		framePosX;
			unsigned int		framePosY;
			unsigned int		retinaColumnStart;
			unsigned int		retinaRowStart;
			unsigned int		retinaRowLength;
			unsigned int		visionPosition;

		// Settings pointers.. performance
			const unsigned int*	retinasperrow;
			const unsigned int*	critter_maxlifetime;
			const unsigned int*	critter_maxenergy;
			const unsigned int*	critter_sightrange;
			const unsigned int*	critter_procinterval;
			const unsigned int*	critter_minenergyproc;
			const unsigned int*	critter_raycastvision;

			const unsigned int*	brain_costhavingneuron;
			const unsigned int*	brain_costfiringneuron;
			const unsigned int*	brain_costfiringmotorneuron;
			const unsigned int*	brain_costhavingsynapse;

			const unsigned int*	condensed_colourmotors;
};

#endif
